@@ -50,10 +50,13 @@ extern int dm_store_enumval(const char *name, int value);
 #ifdef PKG_MALLOC
 #include "../../../dprint.h"
 #define LOG_DBG LM_DBG
+#define LOG_ERROR LM_ERR
 #else
 #define LOG_DBG fd_log_debug
+#define LOG_ERROR fd_log_error
 #endif
 
+
 #define STR_L(s) s, strlen(s)
 #define avp_type2str(t) ( \
 	t == AVP_TYPE_OCTETSTRING ? "string" : \
@@ -468,7 +471,7 @@ int register_osips_avps(void)
 }
 
 
-int parse_avp_def(struct dm_avp_def *avps, int *avp_count, char *line, int len)
+static int parse_avp_def(struct dm_avp_def *avps, int *avp_count, char *line, int len)
 {
 	char *p = line, *avp_name;
 
@@ -477,7 +480,7 @@ int parse_avp_def(struct dm_avp_def *avps, int *avp_count, char *line, int len)
 	avps[*avp_count].name_len = p - avp_name;
 
 	if (avps[*avp_count].name_len > 64) {
-		printf("ERROR: AVP max name length exceeded (64)\n");
+		LOG_ERROR("AVP max name length exceeded (64)\n");
 		return -1;
 	}
 
@@ -521,7 +524,7 @@ int parse_avp_def(struct dm_avp_def *avps, int *avp_count, char *line, int len)
 		break;
 
 	default:
-		printf("ERROR: bad AVP flag in: '... | %s'\n", p);
+		LOG_ERROR("bad AVP flag in: '... | %s'\n", p);
 		goto error;
 	}
 
@@ -535,7 +538,7 @@ int parse_avp_def(struct dm_avp_def *avps, int *avp_count, char *line, int len)
 
 	avps[*avp_count].max_repeats = (int)strtol(p, NULL, 10);
 	if (avps[*avp_count].max_repeats < -1) {
-		printf("ERROR: bad AVP max count: '... | %s'\n", p);
+		LOG_ERROR("bad AVP max count: '... | %s'\n", p);
 		goto error;
 	}
 
@@ -547,11 +550,10 @@ int parse_avp_def(struct dm_avp_def *avps, int *avp_count, char *line, int len)
 	return 0;
 
 error:
-	printf("ERROR: failed to parse line: '%s'\n", line);
+	LOG_ERROR("failed to parse line: '%s'\n", line);
 	return -1;
 }
 
-
 int parse_attr_def(char *line, FILE *fp)
 {
 	struct dm_avp_def avps[128];
@@ -559,8 +561,9 @@ int parse_attr_def(char *line, FILE *fp)
 	unsigned int vendor_id = -1;
 	size_t buflen = strlen(line);
 	int i, len = buflen, attr_len = strlen("ATTRIBUTE"), name_len, avp_code;
-	char *name, *nt_name = NULL, *newp, *p = line, *end = p + len;
+	char *name, *nt_name, *newp, *p = line, *end = p + len;
 	enum dict_avp_basetype avp_type;
+	enum dict_avp_enc_type enc_type = AVP_ENC_TYPE_NONE;
 
 	if (len < attr_len || strncasecmp(p, "ATTRIBUTE", attr_len))
 		return 1;
@@ -578,10 +581,6 @@ int parse_attr_def(char *line, FILE *fp)
 		goto error;
 
 	nt_name = malloc(name_len + 1);
-	if (!nt_name) {
-		printf("ERROR: Malloc failed\n");
-		return -1;
-	}
 	memcpy(nt_name, name, name_len);
 	nt_name[name_len] = '\0';
 
@@ -601,7 +600,14 @@ int parse_attr_def(char *line, FILE *fp)
 	if (p >= end) {
 		avp_type = AVP_TYPE_OCTETSTRING;
 	} else {
-		if ((len >= strlen("utf8string") && !strncasecmp(p, STR_L("utf8string")))
+		if ((len >= strlen("ip") && !strncasecmp(p, STR_L("ip")))) {
+			avp_type = AVP_TYPE_OCTETSTRING;
+			enc_type = AVP_ENC_TYPE_IP;
+		} else if ((len >= strlen("hex") && !strncasecmp(p, STR_L("hex")))
+		        || (len >= strlen("hexstring") && !strncasecmp(p, STR_L("hexstring")))) {
+			avp_type = AVP_TYPE_OCTETSTRING;
+			enc_type = AVP_ENC_TYPE_HEX;
+		} else if ((len >= strlen("utf8string") && !strncasecmp(p, STR_L("utf8string")))
 		        || (len >= strlen("string") && !strncasecmp(p, STR_L("string"))))
 			avp_type = AVP_TYPE_OCTETSTRING;
 		else if ((len >= strlen("unsigned64") && !strncasecmp(p, STR_L("unsigned64"))))
@@ -652,14 +658,12 @@ int parse_attr_def(char *line, FILE *fp)
 			goto create_avp;
 
 		if (avp_count >= 128) {
-			printf("ERROR: max AVP count exceeded (128)\n");
-			free(nt_name);
+			LOG_ERROR("max AVP count exceeded (128)\n");
 			return -1;
 		}
 
 		if (parse_avp_def(avps, &avp_count, p, len) != 0) {
-			printf("ERROR: failed to parse Grouped sub-AVP line: '%s'\n", line);
-			free(nt_name);
+			LOG_ERROR("failed to parse Grouped sub-AVP line: '%s'\n", line);
 			return -1;
 		}
 	}
@@ -667,15 +671,23 @@ int parse_attr_def(char *line, FILE *fp)
 create_avp:;
 	struct dict_object *parent, *avp_ref, **pref;
 
-	if (avp_type == AVP_TYPE_OCTETSTRING) {
+	if (enc_type != AVP_ENC_TYPE_NONE &&
+			dm_enc_add((vendor_id != -1?vendor_id:0), avp_code, enc_type) != 0) {
+		LOG_ERROR("failed to add encoding type\n");
+		return -1;
+	}
+
+	pref = NULL;
+	parent = NULL;
+	switch (avp_type) {
+	case AVP_TYPE_OCTETSTRING:
 		FD_CHECK_dict_search(DICT_TYPE, TYPE_BY_NAME, "UTF8String", &parent);
-		pref = NULL;
-	} else if (avp_type == AVP_TYPE_GROUPED) {
-		parent = NULL;
+		break;
+	case AVP_TYPE_GROUPED:
 		pref = &avp_ref;
-	} else {
-		parent = NULL;
-		pref = NULL;
+		break;
+	default:
+		break;
 	}
 
 	struct dict_avp_data data = {
@@ -697,22 +709,20 @@ create_avp:;
 			DICT_AVP, AVP_BY_NAME_ALL_VENDORS, avps[i].name, &data.rule_avp, 0));
 
 		if (!data.rule_avp) {
-			printf("ERROR: failed to locate AVP: %s\n", avps[i].name);
+			LOG_ERROR("failed to locate AVP: %s\n", avps[i].name);
 			return -1;
 		}
 
 		FD_CHECK_dict_new(DICT_RULE, &data, avp_ref, NULL);
 	}
 
-	LOG_DBG("registered custom AVP (%s, code %d, type %s, sub-avps: %d, vendor: %d)\n",
-			nt_name, avp_code, avp_type2str(avp_type), avp_count, vendor_id);
+	LOG_DBG("registered custom AVP (%s, code %d, type %s, enc %s, sub-avps: %d, vendor: %d)\n",
+			nt_name, avp_code, avp_type2str(avp_type), enc_type2str(enc_type), avp_count, vendor_id);
 
 	free(nt_name);
 	return 0;
 error:
-	printf("ERROR: failed to parse line: %s\n", line);
-	if (nt_name)
-		free(nt_name);
+	LOG_ERROR("failed to parse line: %s\n", line);
 	return -1;
 }
 
@@ -732,7 +742,7 @@ int parse_app_vendor(char *line, FILE *fp)
 
 	vendor_id = (unsigned int)strtoul(p, &newp, 10);
 	if (vendor_id < 0) {
-		printf("ERROR: bad Vendor ID: '... | %s'\n", p);
+		LOG_ERROR("bad Vendor ID: '... | %s'\n", p);
 		return -1;
 	}
 
@@ -740,7 +750,7 @@ int parse_app_vendor(char *line, FILE *fp)
 	p = newp;
 
 	if (len <= 0) {
-		printf("ERROR: empty Vendor Name not allowed\n");
+		LOG_ERROR("empty Vendor Name not allowed\n");
 		return -1;
 	}
 
@@ -772,7 +782,7 @@ int parse_app_def(char *line, FILE *fp)
 	struct dict_object *vendor_dict;
 
 	if (n_app_ids >= 64) {
-		printf("ERROR: max allowed Applications reached (64)\n");
+		LOG_ERROR("max allowed Applications reached (64)\n");
 		return -1;
 	}
 
@@ -800,7 +810,7 @@ int parse_app_def(char *line, FILE *fp)
 
 	app_id = (unsigned int)strtoul(p, &newp, 10);
 	if (app_id < 0) {
-		printf("ERROR: bad Application ID: '... | %s'\n", p);
+		LOG_ERROR("bad Application ID: '... | %s'\n", p);
 		return -1;
 	}
 
@@ -817,7 +827,7 @@ int parse_app_def(char *line, FILE *fp)
 
 		vendor_id = (unsigned int)strtoul(p, &newp, 10);
 		if (vendor_id < 0) {
-			printf("ERROR: bad Vendor ID: '... | %s'\n", p);
+			LOG_ERROR("bad Vendor ID: '... | %s'\n", p);
 			return -1;
 		}
 
@@ -833,7 +843,7 @@ int parse_app_def(char *line, FILE *fp)
 	}
 
 	if (len <= 0) {
-		printf("ERROR: empty Application Name not allowed\n");
+		LOG_ERROR("empty Application Name not allowed\n");
 		return -1;
 	}
 
@@ -891,7 +901,7 @@ int parse_command_def(char *line, FILE *fp, int cmd_type)
 
 	cmd_code = (unsigned int)strtoul(p, &newp, 10);
 	if (cmd_code < 0) {
-		printf("ERROR: bad AVP cmd code: '... | %s'\n", p);
+		LOG_ERROR("bad AVP cmd code: '... | %s'\n", p);
 		return -1;
 	}
 
@@ -908,7 +918,7 @@ int parse_command_def(char *line, FILE *fp, int cmd_type)
 
 	cmd_name_len = p - bkp;
 	if (cmd_name_len > 128) {
-		printf("ERROR: max Command Name length exceeded (128)\n");
+		LOG_ERROR("max Command Name length exceeded (128)\n");
 		return -1;
 	}
 
@@ -930,12 +940,12 @@ int parse_command_def(char *line, FILE *fp, int cmd_type)
 			goto define_req;
 
 		if (avp_count >= 128) {
-			printf("ERROR: max AVP count exceeded (128)\n");
+			LOG_ERROR("max AVP count exceeded (128)\n");
 			return -1;
 		}
 
 		if (parse_avp_def(avps, &avp_count, p, len) != 0) {
-			printf("ERROR: failed to parse Command AVP line: '%s'\n", line);
+			LOG_ERROR("failed to parse Command AVP line: '%s'\n", line);
 			return -1;
 		}
 	}
@@ -962,7 +972,7 @@ define_req:
 			DICT_AVP, AVP_BY_NAME_ALL_VENDORS, avps[i].name, &data.rule_avp, 0));
 
 		if (!data.rule_avp) {
-			printf("ERROR: failed to locate AVP: %s\n", avps[i].name);
+			LOG_ERROR("failed to locate AVP: %s\n", avps[i].name);
 			return -1;
 		}
 
@@ -981,7 +991,7 @@ define_req:
 				DICT_AVP, AVP_BY_NAME, "Transaction-Id", &data.rule_avp, 0));
 
 			if (!data.rule_avp) {
-				printf("ERROR: failed to locate Transaction-Id AVP\n");
+				LOG_ERROR("failed to locate Transaction-Id AVP\n");
 				return -1;
 			}
 
@@ -1001,7 +1011,7 @@ define_req:
 				DICT_AVP, AVP_BY_NAME, "Result-Code", &data.rule_avp, 0));
 
 			if (!data.rule_avp) {
-				printf("ERROR: failed to locate Result-Code AVP\n");
+				LOG_ERROR("failed to locate Result-Code AVP\n");
 				return -1;
 			}
 
@@ -1084,7 +1094,7 @@ int parse_extra_avps(const char *extra_avps_file)
 	}
 
 	if (answers_needed > 0) {
-		printf("ERROR: bad config file, at least one Diameter Answer "
+		LOG_ERROR("bad config file, at least one Diameter Answer "
 		       "definition is missing\n");
 		ret = -1;
 	}
