@@ -1674,8 +1674,8 @@ static int b2b_sdp_demux_start(struct sip_msg *msg, str *uri,
 	struct list_head *it;
 	struct b2b_sdp_client *client;
 	client_info_t ci;
-	struct socket_info *si;
-	str *sess_ip;
+	const struct socket_info *si;
+	const str *sess_ip;
 
 	if (!msg->force_send_socket) {
 		si = uri2sock(msg, uri, &tmp_su, PROTO_NONE);
@@ -1707,10 +1707,8 @@ static int b2b_sdp_demux_start(struct sip_msg *msg, str *uri,
 		LM_ERR("could not copy b2b server key\n");
 		/* key is not yet stored, so cannot be deleted */
 		b2b_api.entity_delete(B2B_SERVER, b2b_key, ctx->dlginfo, 1, 1);
-		pkg_free(b2b_key);
 		return -1;
 	}
-	pkg_free(b2b_key);
 	/* we need to wait for all pending clients */
 	ctx->pending_no = ctx->clients_no;
 
