@@ -76,7 +76,7 @@ static void dlg_update_req_info(str *buffer, struct dlg_cell *dlg, int leg,
 
 
 static void dlg_update_ack_sdp(struct sip_msg* req, str *buffer, int rpl_code,
-				union sockaddr_union *to, struct socket_info *sock, int proto)
+				const union sockaddr_union *to, const struct socket_info *sock, int proto)
 {
 	struct dlg_cell *dlg;
 
@@ -1479,7 +1479,6 @@ static inline int dlg_update_contact(struct dlg_cell *dlg, struct sip_msg *msg,
 	str contact, new_ct, old_ct;
 	int ret = 0;
 	contact_t *ct = NULL;
-	contact_body_t *parsed_body = NULL;
 
 	if (!msg->contact &&
 		(parse_headers(msg, HDR_CONTACT_F, 0) < 0 || !msg->contact)) {
@@ -1497,14 +1496,7 @@ static inline int dlg_update_contact(struct dlg_cell *dlg, struct sip_msg *msg,
 		contact = ct->uri;
 		LM_DBG("Found unparsed contact [%.*s]\n", contact.len, contact.s);
 	} else {
-		parsed_body = (contact_body_t *)msg->contact->parsed;
-
-		if (parsed_body->star == 1) {
-			LM_WARN("Invalid star Contact in dialog update!\n");
-			return 0;
-		}
-
-		contact = parsed_body->contacts->uri;
+		contact = ((contact_body_t *)msg->contact->parsed)->contacts->uri;
 	}
 
 	/* if the same contact, don't do anything */
@@ -2262,8 +2254,6 @@ after_unlock5:
 					req->msg_flags |= FL_USE_UAC_CSEQ;
 				}
 
-				if (dialog_repl_cluster)
-					replicate_dialog_cseq_updated(dlg, dst_leg);
 				dlg_unlock( d_table, d_entry );
 
 				LM_DBG("nonACK req [%d],NULL,%d)\n",
@@ -2303,8 +2293,6 @@ after_unlock5:
 				else {
 					LM_DBG("update_val=[%d]\n", update_val);
 				}
-				if (dialog_repl_cluster)
-					replicate_dialog_cseq_updated(dlg, dst_leg);
 				dlg_unlock( d_table, d_entry );
 
 				if (update_val && update_msg_cseq(req,0,update_val) != 0) {
