@@ -48,10 +48,15 @@ str shtag_dlg_val = str_init("dlgX_shtag");
 char *dlg_sync_in_progress;
 
 static int get_shtag_sync_status(struct dlg_cell *dlg);
+/*
+ * indicates whether the dialog is in the process of receiving a replicated
+ * dialog (update) - used to avoid cross-replicating the same value
+ */
+static int dlg_event_is_replicated = 0;
 
-static struct socket_info * fetch_socket_info(str *addr)
+const static struct socket_info * fetch_socket_info(str *addr)
 {
-	struct socket_info *sock;
+	const struct socket_info *sock;
 	if (!addr || !addr->len)
 		return NULL;
 	sock = parse_sock_info(addr);
@@ -97,6 +102,9 @@ static struct dlg_cell *lookup_dlg_unsafe(unsigned int h_entry, unsigned int h_i
 
 	for( dlg=d_entry->first ; dlg ; dlg=dlg->next ) {
 		if (dlg->h_id == h_id) {
+			if (dlg->state==DLG_STATE_DELETED)
+				goto not_found;
+
 			LM_DBG("dialog id=%u found on entry %u\n", h_id, h_entry);
 			return dlg;
 		}
@@ -173,7 +181,7 @@ int dlg_replicated_create(bin_packet_t *packet, struct dlg_cell *cell,
 	str sdp1, sdp2, sdp3, sdp4;
 	str sock, vars, profiles;
 	struct dlg_cell *dlg = NULL;
-	struct socket_info *caller_sock, *callee_sock;
+	const struct socket_info *caller_sock, *callee_sock;
 	struct dlg_entry *d_entry;
 	int_str tag_name;
 	unsigned int h_id;
@@ -346,8 +354,6 @@ int dlg_replicated_create(bin_packet_t *packet, struct dlg_cell *cell,
 				NULL, DLG_DIR_NONE, -1, NULL, 1, 0);
 	}
 
-	dlg->locked_by = process_no;
-
 	if ((rc = fetch_dlg_value(dlg, &shtag_dlg_val, &dlg_val_type, &tag_name, 0)) == 0) {
 		if (dlg_val_type != DLG_VAL_TYPE_STR) {
 			LM_ERR("Bad dialog value type\n");
@@ -358,8 +364,6 @@ int dlg_replicated_create(bin_packet_t *packet, struct dlg_cell *cell,
 		LM_ERR("Failed to get dlg value for sharing tag %.*s\n",
 		       tag_name.s.len, tag_name.s.s);
 
-	dlg->locked_by = 0;
-
 	if (from_sync) {
 		dlg->flags |= DLG_FLAG_SYNCED;
 		/* drop a dialog that is not marked with a sharing tag we're
@@ -692,40 +696,92 @@ int dlg_replicated_cseq_updated(bin_packet_t *packet)
 
 	LM_DBG("Updating cseq for dialog with callid: %.*s\n", call_id.len, call_id.s);
 
-	if (pkg_ver == DLG_BIN_V4)
+	if (pkg_ver == DLG_BIN_V4) {
 		DLG_BIN_POP(int, packet, h_id, malformed);
-	DLG_BIN_POP(int, packet, cseq, malformed);
 
-	h_entry = dlg_hash(&call_id);
-	d_entry = &(d_table->entries[h_entry]);
-	dlg_lock(d_table, d_entry);
+		h_entry = dlg_hash(&call_id);
+		d_entry = &(d_table->entries[h_entry]);
+
+		dlg_lock(d_table, d_entry);
 
-	if (pkg_ver == DLG_BIN_V4)
 		dlg = lookup_dlg_unsafe(h_entry, h_id);
-	else
-		get_dlg_unsafe(d_entry, &call_id, &from_tag, &to_tag, &dlg);
+
+		if (dlg && !match_dialog(dlg, &call_id, &from_tag, &to_tag, &dir, &dst_leg)) {
+			LM_ERR("Failed to match dialog\n");
+			dlg_unlock(d_table, d_entry);
+			return -1;
+		}
+
+		dlg_unlock(d_table, d_entry);
+	} else {
+		dlg = get_dlg(&call_id, &from_tag, &to_tag, &dir, &dst_leg);
+	}
 
 	if (!dlg) {
 		/* may be already deleted due to timeout */
 		LM_DBG("dialog not found (callid: |%.*s| ftag: |%.*s|\n",
 			call_id.len, call_id.s, from_tag.len, from_tag.s);
-		goto unlock;
+		return 0;
 	}
 
-	if (dlg->state == DLG_STATE_DELETED) {
-		LM_DBG("dialog already deleted (callid: |%.*s| ftag: |%.*s|\n",
-				call_id.len, call_id.s, from_tag.len, from_tag.s);
-		goto unlock;
-	}
-
-	if (!match_dialog(dlg, &call_id, &from_tag, &to_tag, &dir, &dst_leg)) {
-		LM_ERR("Failed to match dialog leg (callid: |%.*s| ftag: |%.*s|\n",
-				call_id.len, call_id.s, from_tag.len, from_tag.s);
-		goto unlock;
-	}
+	DLG_BIN_POP(int, packet, cseq, malformed);
 	dlg->legs[dst_leg].last_gen_cseq = cseq;
 
-unlock:
+	if (pkg_ver != DLG_BIN_V4)
+		unref_dlg(dlg, 1);
+
+	return 0;
+malformed:
+	LM_ERR("malformed cseq update packet for %.*s\n", call_id.len, call_id.s);
+	return -1;
+}
+
+int dlg_replicated_value(bin_packet_t *packet)
+{
+	str call_id, name;
+	struct dlg_cell *dlg;
+	unsigned int h_id;
+	int h_entry, type, ret;
+	struct dlg_entry *d_entry;
+	int_str val;
+
+	DLG_BIN_POP(str, packet, call_id, malformed);
+	DLG_BIN_POP(int, packet, h_id, malformed);
+	DLG_BIN_POP(str, packet, name, malformed);
+	DLG_BIN_POP(int, packet, type, malformed);
+	switch (type) {
+		case DLG_VAL_TYPE_STR:
+			DLG_BIN_POP(str, packet, val.s, malformed);
+			break;
+		case DLG_VAL_TYPE_INT:
+			DLG_BIN_POP(int, packet, val.n, malformed);
+			break;
+		default:
+			break;
+	}
+
+	LM_DBG("Updating cseq for dialog with callid: %.*s\n", call_id.len, call_id.s);
+	h_entry = dlg_hash(&call_id);
+	d_entry = &(d_table->entries[h_entry]);
+
+	dlg_lock(d_table, d_entry);
+
+	dlg = lookup_dlg_unsafe(h_entry, h_id);
+	if (!dlg) {
+		LM_DBG("unable to find dialog %.*s [%u:%d]\n",
+				call_id.len, call_id.s, h_id, h_entry);
+		dlg_unlock(d_table, d_entry);
+		return -1;
+	}
+	lock_start_write(dlg->vals_lock);
+	ret = store_dlg_value_unsafe(dlg, &name, &val, type);
+	lock_stop_write(dlg->vals_lock);
+	if (ret < 0)
+		LM_ERR("cannot store dlg value\n");
+	else
+		run_dlg_callbacks(DLGCB_PROCESS_VARS, dlg,
+				NULL, DLG_DIR_NONE, -1, &name, 1, 0);
+
 	dlg_unlock(d_table, d_entry);
 
 	return 0;
@@ -792,12 +848,12 @@ void bin_push_dlg(bin_packet_t *packet, struct dlg_cell *dlg)
 
    /* save sharing tag name as dlg val */
 	isval.s = dlg->shtag;
-	if (dlg->shtag.s && store_dlg_value_unsafe(dlg, &shtag_dlg_val, &isval,
+	if (dlg->shtag.s && store_dlg_value(dlg, &shtag_dlg_val, &isval,
 		DLG_VAL_TYPE_STR) < 0)
 		LM_ERR("Failed to store sharing tag %.*s(%p) as dlg val\n",
 		       dlg->shtag.len, dlg->shtag.s, dlg->shtag.s);
 
-	vars = write_dialog_vars(dlg->vals);
+	vars = write_dialog_vars(dlg);
 	profiles = write_dialog_profiles(dlg->profile_links);
 
 	bin_push_str(packet, vars);
@@ -817,6 +873,25 @@ void bin_push_dlg(bin_packet_t *packet, struct dlg_cell *dlg)
 
 /*  Binary Packet sending functions   */
 
+#define DLG_CLUSTER_SEND(packet, dialog_repl_cluster, error) \
+	do { \
+		int rc; \
+		rc = clusterer_api.send_all(&packet, dialog_repl_cluster); \
+		switch (rc) { \
+		case CLUSTERER_CURR_DISABLED: \
+			LM_INFO("Current node is disabled in cluster: %d\n", dialog_repl_cluster); \
+			goto error; \
+		case CLUSTERER_DEST_DOWN: \
+			LM_INFO("All destinations in cluster: %d are down or probing\n", \
+				dialog_repl_cluster); \
+			goto error; \
+		case CLUSTERER_SEND_ERR: \
+			LM_ERR("Error sending in cluster: %d\n", dialog_repl_cluster); \
+			goto error; \
+		} \
+	} while(0)
+
+
 
 /**
  * replicates a locally created dialog to all the destinations
@@ -824,7 +899,6 @@ void bin_push_dlg(bin_packet_t *packet, struct dlg_cell *dlg)
  */
 void replicate_dialog_created(struct dlg_cell *dlg)
 {
-	int rc;
 	bin_packet_t packet;
 
 	dlg_lock_dlg(dlg);
@@ -855,19 +929,7 @@ void replicate_dialog_created(struct dlg_cell *dlg)
 
 	dlg_unlock_dlg(dlg);
 
-	rc = clusterer_api.send_all(&packet, dialog_repl_cluster);
-	switch (rc) {
-	case CLUSTERER_CURR_DISABLED:
-		LM_INFO("Current node is disabled in cluster: %d\n", dialog_repl_cluster);
-		goto error;
-	case CLUSTERER_DEST_DOWN:
-		LM_INFO("All destinations in cluster: %d are down or probing\n",
-			dialog_repl_cluster);
-		goto error;
-	case CLUSTERER_SEND_ERR:
-		LM_ERR("Error sending in cluster: %d\n", dialog_repl_cluster);
-		goto error;
-	}
+	DLG_CLUSTER_SEND(packet, dialog_repl_cluster, error);
 
 	if_update_stat(dlg_enable_stats,create_sent,1);
 	bin_free_packet(&packet);
@@ -890,10 +952,8 @@ no_send:
  */
 void replicate_dialog_updated(struct dlg_cell *dlg)
 {
-	int rc;
 	bin_packet_t packet;
 
-
 	dlg_lock_dlg(dlg);
 	if (dlg->state < DLG_STATE_CONFIRMED_NA) {
 		LM_DBG("not replicating update in state %d (%.*s)\n", dlg->state,
@@ -919,19 +979,7 @@ void replicate_dialog_updated(struct dlg_cell *dlg)
 
 	dlg_unlock_dlg(dlg);
 
-	rc = clusterer_api.send_all(&packet, dialog_repl_cluster);
-	switch (rc) {
-	case CLUSTERER_CURR_DISABLED:
-		LM_INFO("Current node is disabled in cluster: %d\n", dialog_repl_cluster);
-		goto error;
-	case CLUSTERER_DEST_DOWN:
-		LM_ERR("All destinations in cluster: %d are down or probing\n",
-			dialog_repl_cluster);
-		goto error;
-	case CLUSTERER_SEND_ERR:
-		LM_ERR("Error sending in cluster: %d\n", dialog_repl_cluster);
-		goto error;
-	}
+	DLG_CLUSTER_SEND(packet, dialog_repl_cluster, error);
 
 	if_update_stat(dlg_enable_stats,update_sent,1);
 	bin_free_packet(&packet);
@@ -954,7 +1002,6 @@ end:
  */
 void replicate_dialog_deleted(struct dlg_cell *dlg)
 {
-	int rc;
 	bin_packet_t packet;
 
 	if (bin_init(&packet, &dlg_repl_cap, REPLICATION_DLG_DELETED, BIN_VERSION, 1024) != 0)
@@ -965,19 +1012,7 @@ void replicate_dialog_deleted(struct dlg_cell *dlg)
 	bin_push_str(&packet, &dlg->legs[callee_idx(dlg)].tag);
 	bin_push_int(&packet, dlg->h_id);
 
-	rc = clusterer_api.send_all(&packet, dialog_repl_cluster);
-	switch (rc) {
-	case CLUSTERER_CURR_DISABLED:
-		LM_INFO("Current node is disabled in cluster: %d\n", dialog_repl_cluster);
-		goto error_free;
-	case CLUSTERER_DEST_DOWN:
-		LM_ERR("All destinations in cluster: %d are down or probing\n",
-			dialog_repl_cluster);
-		goto error_free;
-	case CLUSTERER_SEND_ERR:
-		LM_ERR("Error sending in cluster: %d\n", dialog_repl_cluster);
-		goto error_free;
-	}
+	DLG_CLUSTER_SEND(packet, dialog_repl_cluster, error_free);
 
 	if_update_stat(dlg_enable_stats, delete_sent, 1);
 	bin_free_packet(&packet);
@@ -994,7 +1029,6 @@ error:
  */
 void replicate_dialog_cseq_updated(struct dlg_cell *dlg, int leg)
 {
-	int rc;
 	bin_packet_t packet;
 
 	if (bin_init(&packet, &dlg_repl_cap, REPLICATION_DLG_CSEQ,
@@ -1009,19 +1043,7 @@ void replicate_dialog_cseq_updated(struct dlg_cell *dlg, int leg)
 
 	bin_push_int(&packet, dlg->legs[leg].last_gen_cseq);
 
-	rc = clusterer_api.send_all(&packet, dialog_repl_cluster);
-	switch (rc) {
-	case CLUSTERER_CURR_DISABLED:
-		LM_INFO("Current node is disabled in cluster: %d\n", dialog_repl_cluster);
-		goto error_free;
-	case CLUSTERER_DEST_DOWN:
-		LM_ERR("All destinations in cluster: %d are down or probing\n",
-			dialog_repl_cluster);
-		goto error_free;
-	case CLUSTERER_SEND_ERR:
-		LM_ERR("Error sending in cluster: %d\n", dialog_repl_cluster);
-		goto error_free;
-	}
+	DLG_CLUSTER_SEND(packet, dialog_repl_cluster, error_free);
 
 	bin_free_packet(&packet);
 	return;
@@ -1031,6 +1053,46 @@ error:
 	LM_ERR("Failed to replicate dialog cseq update\n");
 }
 
+void replicate_dialog_value(struct dlg_cell *dlg, str *name, int_str *val, int type)
+{
+	bin_packet_t packet;
+
+	if (dlg_event_is_replicated)
+		return;
+
+	if (bin_init(&packet, &dlg_repl_cap, REPLICATION_DLG_VALUE,
+			BIN_VERSION, 512) != 0)
+		goto error;
+
+	bin_push_str(&packet, &dlg->callid);
+	bin_push_int(&packet, dlg->h_id);
+	bin_push_str(&packet, name);
+	bin_push_int(&packet, type);
+	if (!val)
+		type = DLG_VAL_TYPE_NONE;
+	switch (type) {
+		case DLG_VAL_TYPE_STR:
+			bin_push_str(&packet, &val->s);
+			break;
+		case DLG_VAL_TYPE_INT:
+			bin_push_int(&packet, val->n);
+			break;
+		default:
+			break;
+	}
+
+	DLG_CLUSTER_SEND(packet, dialog_repl_cluster, error_free);
+
+	bin_free_packet(&packet);
+	return;
+error_free:
+	bin_free_packet(&packet);
+error:
+	LM_ERR("Failed to replicate dialog values\n");
+}
+
+#undef DLG_CLUSTER_SEND
+
 void receive_dlg_repl(bin_packet_t *pkt)
 {
 	int rc = 0;
@@ -1042,6 +1104,7 @@ void receive_dlg_repl(bin_packet_t *pkt)
 		if (ver != DLG_BIN_V3)
 			ensure_bin_version(pkt, BIN_VERSION);
 
+		dlg_event_is_replicated = 1;
 		rc = dlg_replicated_create(pkt, NULL, NULL, NULL, 0, 0, 0);
 		if_update_stat(dlg_enable_stats, create_recv, 1);
 		break;
@@ -1049,6 +1112,7 @@ void receive_dlg_repl(bin_packet_t *pkt)
 		if (ver != DLG_BIN_V3)
 			ensure_bin_version(pkt, BIN_VERSION);
 
+		dlg_event_is_replicated = 1;
 		rc = dlg_replicated_update(pkt);
 		if_update_stat(dlg_enable_stats, update_recv, 1);
 		break;
@@ -1056,6 +1120,7 @@ void receive_dlg_repl(bin_packet_t *pkt)
 		if (ver != DLG_BIN_V3)
 			ensure_bin_version(pkt, BIN_VERSION);
 
+		dlg_event_is_replicated = 1;
 		rc = dlg_replicated_delete(pkt);
 		if_update_stat(dlg_enable_stats, delete_recv, 1);
 		break;
@@ -1063,15 +1128,24 @@ void receive_dlg_repl(bin_packet_t *pkt)
 		if (ver != DLG_BIN_V3)
 			ensure_bin_version(pkt, BIN_VERSION);
 
+		dlg_event_is_replicated = 1;
 		rc = dlg_replicated_cseq_updated(pkt);
 		break;
+	case REPLICATION_DLG_VALUE:
+		ensure_bin_version(pkt, BIN_VERSION);
+
+		dlg_event_is_replicated = 1;
+		rc = dlg_replicated_value(pkt);
+		break;
 	case SYNC_PACKET_TYPE:
 		if (ver != DLG_BIN_V3)
 			ensure_bin_version(pkt, BIN_VERSION);
 
+		dlg_event_is_replicated = 1;
 		while (clusterer_api.sync_chunk_iter(pkt))
 			if (dlg_replicated_create(pkt, NULL, NULL, NULL, 0, 0, 1) < 0) {
 				LM_ERR("Failed to process sync packet\n");
+				dlg_event_is_replicated = 0;
 				return;
 			}
 		break;
@@ -1084,6 +1158,7 @@ void receive_dlg_repl(bin_packet_t *pkt)
 
 	if (rc != 0)
 		LM_ERR("Failed to process a binary packet!\n");
+	dlg_event_is_replicated = 0;
 }
 
 static int receive_sync_request(int node_id)
