@@ -1,27 +1,27 @@
 /**
-* Topology Hiding Module
-*
-* Copyright (C) 2015 OpenSIPS Foundation
-*
-* This file is part of opensips, a free SIP server.
-*
-* opensips is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation; either version 2 of the License, or
-* (at your option) any later version
-*
-* opensips is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-*
-* You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
-*
-* History
-* -------
-*  2015-02-17  initial version (Vlad Paiu)
+ * Topology Hiding Module
+ *
+ * Copyright (C) 2015 OpenSIPS Foundation
+ *
+ * This file is part of opensips, a free SIP server.
+ *
+ * opensips is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version
+ *
+ * opensips is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *
+ * History
+ * -------
+ *  2015-02-17  initial version (Vlad Paiu)
 */
 
 #include "../../ut.h"
@@ -38,8 +38,8 @@ extern str th_contact_encode_param;
 extern int th_ct_enc_scheme;
 
 struct th_ct_params {
-str param_name;
-struct th_ct_params *next;
+	str param_name;
+	struct th_ct_params *next;
 };
 static struct th_ct_params *th_param_list=NULL;
 static struct th_ct_params *th_hdr_param_list=NULL;
@@ -52,7 +52,7 @@ static int topo_delete_record_routes(struct sip_msg *req);
 static struct lump* delete_existing_contact(struct sip_msg *msg, int del_hdr);
 static int topo_parse_passed_params(str *params,struct th_ct_params **lst);
 static void topo_dlg_onroute (struct dlg_cell* dlg, int type,
-	struct dlg_cb_params * params);
+		struct dlg_cb_params * params);
 static void topo_dlg_initial_reply (struct dlg_cell* dlg, int type,
 		struct dlg_cb_params * params);
 static void th_down_onreply(struct cell* t, int type,struct tmcb_params *param);
@@ -1846,7 +1846,7 @@ static int topo_no_dlg_seq_handling(struct sip_msg *msg,str *info)
 	struct lump* lmp = NULL;
 	str host;
 	int port,proto;
-	struct socket_info *sock;
+	const struct socket_info *sock;
 	str *route_s = NULL, route_buf = {0, 0};
 	transaction_cb* used_cb;
 
